@@ -3,6 +3,7 @@ from collections import OrderedDict
 import datetime
 import logging
 import os
+import re
 import sqlite3
 import sys
 import uuid
@@ -20,15 +21,15 @@ def note(exclude=None, actual=False):
     model = [('uid', 'string'),
              ('created', 'string'),
              ('updated', 'string'),
-             ('subject', None),
+             ('subject', 'string'),
              ('tags', 'string'),
              ('content', 'string'),
+             ('encrypted', 'integer'),
             ]
     n = OrderedDict((k, v) for k, v in model if not k in exclude)
     if actual:
-        now = datetime.datetime.now().strftime('%Y-%m-%d %H:%M')
         uid = uuid.uuid4().hex
-        n.update(uid=uid, created=now, updated=now)
+        n.update(uid=uid, created=now(), updated=now())
     return n
 
 def create_note(n, password=None):
@@ -41,42 +42,64 @@ def create_note(n, password=None):
     values = [v for v in n.values() if not v is None]
     log.debug('%s, %s' % (sql, values))
     c.execute(sql, values)
-    return c.commit()
+    c.commit()
+    n.pop('content')
+    return n
 
-def delete_note(uid, password=None):
+def delete_note(uid):
     c = conn()
     sql = 'DELETE FROM notes WHERE uid = ?'
     log.debug('%s, %s' % (sql, uid))
     c.execute(sql, (uid,))
-    return c.commit()
+    c.commit()
+    return True
 
 def encrypt(n, password):
+    encrypted = False
+    content = n['content']
     if password:
-        subject = n['content'].split('\n')[0]
-        n['content'] = '\n'.join((subject,
-                                  crypt.encrypt(n['content'], password)))
+        content = crypt.encrypt(content, password)
+        encrypted = True
+    n.update(dict(content=content, encrypted=encrypted))
     return n
 
+def now():
+    return datetime.datetime.now().strftime('%Y-%m-%d %H:%M')
+
 def update_note(n, password=None):
     c = conn()
     n = encrypt(n, password)
-    sql = 'UPDATE notes SET tags = ?, content = ?, updated = ? WHERE uid = ?'
-    values = [n.get('tags'), n.get('content'), n.get('updated'), n.get('uid')]
+    n['updated'] = now()
+    sql = """
+      UPDATE notes SET
+        tags = ?,
+        subject = ?,
+        content = ?,
+        encrypted = ?,
+        updated = ?
+      WHERE uid = ?
+      """
+    values = [n.get('tags'),
+              n.get('subject'),
+              n.get('content'),
+              n.get('encrypted'),
+              n.get('updated'),
+              n.get('uid')]
     log.debug('%s, %s' % (sql, values))
     c.execute(sql, values)
-    return c.commit()
+    c.commit()
+    n.pop('content')
+    return n
 
 def conn():
     d = os.path.dirname(path)
     _ = os.makedirs(d) if not os.path.exists(d) else None
     return sqlite3.connect(path)
 
-def calculate_subject(row):
-    return row.get('content').split('\n')[0]
-
 def columns(n, row):
+    # TODO: This can be deleted
     for k, _ in n.items():
-        yield dict(v=row.get(k, getattr(mod, 'calculate_subject')(row)))
+        yield row.get(k)
 
 def dict_factory(c, row):
     return dict((col[0], row[i]) for i, col in enumerate(c.description))
@@ -87,7 +110,7 @@ def fields(n):
 
 def rows(n, rs):
     for row in rs:
-        yield dict(c=list(columns(n, row)))
+        yield list(columns(n, row))
 
 def create_schema(c):
     pairs = [' '.join(pair) for pair in note().items() if pair[1]]
@@ -102,21 +125,57 @@ def get_content(uid, password):
     c.row_factory = dict_factory
     sql = 'SELECT content FROM notes WHERE uid = ?'
     content = c.cursor().execute(sql, [uid]).next().get('content')
-    return crypt.decrypt(content.split('\n', 1)[1], password)
+    return crypt.decrypt(content, password) if password else content
 
-def search(s):
+def migrate_data(c):
+    # Migrate 1 encrypted contents to column + dedicated content block
+    sql ="""
+        UPDATE notes SET
+            subject = ?,
+            content = ?,
+            encrypted = ?
+        WHERE uid = ?
+    """
+    c.rollback()
+    for n in search(''):
+        if n['subject']:
+            continue
+        encrypted = False
+        content = n['content'].split('\n')
+        if len(content) > 1 and re.search(r'([^ ]{32,})', content[1]):
+            encrypted = True
+        values = (content.pop(0), '\n'.join(content).strip(), encrypted, n['uid'])
+        c.execute(sql, values)
+        log.debug('Migrated: %s', values[0])
+    c.commit()
+
+def migrate_schema(c):
+    sqls =['ALTER TABLE notes ADD COLUMN encrypted INTEGER DEFAULT 0;',
+           ' ALTER TABLE notes ADD COLUMN subject TEXT;']
+    try:
+        for sql in sqls:
+            c.execute(sql)
+    except sqlite3.OperationalError as ex:
+        log.debug('schema migration: %s', ex)
+
+def search(s, exclude=None):
     terms = s.split() if s else []
-    n = note()
+    n = note(exclude=exclude)
     naive = "(content LIKE '%{0}%' OR tags LIKE '%{0}%')"
     where = ['1=1'] + [naive.format(t) for t in terms]
-    sql = 'SELECT %s FROM notes WHERE %s;'
+    sql = 'SELECT %s FROM notes WHERE %s ORDER BY updated DESC;'
     sql = sql % (','.join(k for k, v in n.items() if v), ' AND '.join(where))
     log.debug(sql)
     c = conn()
     c.row_factory = dict_factory
-    _cols = list(fields(n))
-    _rows = list(rows(n, c.cursor().execute(sql)))
-    return dict(cols=_cols, rows=_rows)
+    cols = list(fields(n))
+    for row in rows(n, c.cursor().execute(sql)):
+        n = {}
+        for i, col in enumerate(cols):
+            n[cols[i]['id']] = row[i]
+        yield n
 
 def prepare():
     create_schema(conn())
+    migrate_schema(conn())
+    migrate_data(conn())
